<?php

// http://www.blitter.com/~russtopia/MIDI/~jglatt/tech/miditech.htm

require_once("binary.php");

function midi_int($x) {
	$result = chr($x & 0x7F);
	while ($x >>= 7) {
		$result = chr(($x & 0x7F) | 0x80) . $result;
	}
	return $result;
}

function midi_chunk($type, $data) {
	assert(strlen($type) === 4);
	return $type . int4mac2str(strlen($data)) . $data;
}

class MIDI_Track {
	var $delay = 0; // Amount of delay since the last message, in PPQN ticks.
	var $result = "";
	var $channel = 0; // Which MIDI channel to send messages to.
	var $length = 0; // Total length, in PPQN ticks.
	/*
	function MIDI_Track($channel) {
		$this->channel = ($channel & 0x0F);
	}
	function __construct($channel) {
		$this->channel = ($channel & 0x0F);
	}
	*/
	function delay($x) {
		$this->delay += $x;
		$this->length += $x;
	}
	function message($s) {
		$this->result .= midi_int($this->delay) . $s;
		$this->delay = 0;
	}
	function note_off($note, $velocity = 64) {
		$this->message(chr(0x80 + $this->channel) . chr($note & 0x7F) . chr($velocity & 0x7F));
	}
	function note_on($note, $velocity = 64) {
		$this->message(chr(0x90 + $this->channel) . chr($note & 0x7F) . chr($velocity & 0x7F));
	}
	function controller($contr, $value) {
		$this->message(chr(0xB0 + $this->channel) . chr($contr & 0x7F) . chr($value & 0x7F));
	}
	function program($x) {
		$this->message(chr(0xC0 + $this->channel) . chr($x & 0x7F));
	}
	function get_result() {
		return $this->result;
	}
	function meta_event($type, $data) {
		$this->message("\xFF" . chr($type & 0x7F) . midi_int(strlen($data)) . $data);
	}
	function tempo($x) {
		//$this->message("\xFF\x51\x03" . substr(int4mac2str($x),1));
		$this->meta_event(0x51, substr(int4mac2str($x),1,3));
	}
	function end() {
		//$this->message("\xFF\x2F\x00");
		$this->meta_event(0x2F, "");
	}
}

function make_MIDI($type, $PPQN, $tracks) {
	$MIDI = "";
	$MThd = 
		int2mac2str($type) .
		int2mac2str(count($tracks)) .
		int2mac2str($PPQN);
	$MIDI .= midi_chunk("MThd",$MThd);
	foreach ($tracks as $MTrk) {
		$MIDI .= midi_chunk("MTrk",$MTrk);
	}
	return $MIDI;
}
